# ROS2串口例程

本文档介绍如何在ROS2下来读取IMU的数据，并提供了c++语言例程代码，通过执行ROS2命令，运行相应的节点，就可以看到打印到终端上的信息。

* 测试环境：Ubuntu22,Ubuntu20

* ROS版本：ROS2 humble
* ROS版本：ROS2 foxy

* 测试设备： nav680

## 安装USB-UART驱动

Ubuntu 认不需要安装串口驱动。将调试版连接到电脑上时，会自动识别设备。识别成功后，会在dev目录下出现一个对应的设备:ttyUSBx

检查USB-UART设备是否被Ubantu识别：

1. 打开终端，输入`ls /dev`,先查看已经存在的串口设备。
2. 查看是否已经存在  ttyUSBx 这个设备文件，便于确认对应的端口号。
4. 接下来插入USB线，连接调试板，然后再次执行`ls /dev`。 dev目录下多了一个设备`ttyUSB0`：

```shell
linux@ubuntu:~$ ls /dev
.....
hpet             net           tty11     tty4   ttyS0      ttyUSB0    vhost-vsock
hugepages        null          tty12     tty40  ttyS1      udmabuf  vmci
......
```

4.打开USB设备的可执行权限：

```shell
   $ sudo chmod 777 /dev/ttyUSB0
```

## 编译imu_ros2工作空间

1. 打开终端进入/forsense_driver/NAV680_ros2目录
2. 执行`colcon build`命令，编译成功后出现如下信息。

```shell
linux@ubuntu20:~/forsense_driver/nav680_ros2$ colcon build
Starting >>> imu_ros2
Finished <<< imu_ros2 [0.24s]                  

Summary: 1 package finished [0.35s]
linux@ubuntu20:~/serial_imu_ws$ 
```

## 3. 修改串口波特率和设备号

1. 在Ubuntu环境中，支持的波特率为115200, 460800, 921600。本例程使用的默认波特率是460800，默认打开的串口名称是/dev/ttyUSB0。	

2. 如果您需要如需要修改波特率，请修改`config/hipnuc_config.yaml`文件中的配置参数。
  注意:imu设备的输出波特率应与ROS2驱动设置的波特率保持一致	

```c
IMU_publisher:
    ros__parameters:
        serial_port: "/dev/ttyUSB0"
        baud_rate: 460800
        frame_id: "base_link"
        imu_topic: "/IMU_data"
```

注意修改后需要回到nav680_ros2目录下，重新执行`colcon build`命令

## 查看NAVha话题消息结构
本例程提供了一种查看数据结构方式：
```shell
linux@ubuntu20:~/forsense_driver/nav680_ros2$  source ./install/setup.bash
linux@ubuntu20:~/forsense_driver/nav680_ros2$  ros2 interface show nav680_ros2/msg/Nav
```
​	2.执行成功后，就可以看到所有的信息：
```shell
std_msgs/Header header
	builtin_interfaces/Time stamp
		int32 sec
		uint32 nanosec
	string frame_id
uint32 itow
uint16 week_num
int32 lat
int32 lon
int32 hgt
float32 vn
float32 ve
float32 vd
float32 roll
float32 pitch
float32 yaw
float32 rtk_yaw
float32 wheel_angle
float32[7] imu
uint8 fix_type
uint8 sv_num
uint8 diff_age
uint8 heading_type
uint16 pos_acc
uint16 status
```



###  输出NAV680话题消息

1. 打开终端，执行：

```shell
linux@ubuntu20:~/forsense_driver/nav680_ros2$  source ./install/setup.bash
linux@ubuntu20:~/forsense_driver/nav680_ros2$ ros2 launch nav680_ros2 imu_spec_msg.launch.py
```



​	2.执行成功后，就可以看到所有的信息：

```c
[listener-2] header:
[listener-2] 	stamp:
[listener-2] 	  secs:1715316791
[listener-2] 	  nanosecs:210183923
[listener-2] 	frame_id:base_link
[listener-2] 	itow: 1574020
[listener-2] 	week_num: 0
[listener-2] 	lat: 0
[listener-2] 	lon: 0
[listener-2] 	hgt: 0
[listener-2] 	vn: 0.000000000000000000
[listener-2] 	ve: 0.000000000000000000
[listener-2] 	vd: 0.000000000000000000
[listener-2] 	roll: -0.633964359760284424
[listener-2] 	pitch: 0.006382254417985678
[listener-2] 	yaw: 0.000000000000000000
[listener-2] 	rtk_yaw: 0.134229674935340881
[listener-2] 	wheel_angle: 0.000000000000000000
[listener-2] 	fix_type: 
[listener-2] 	heading_type: 
[listener-2] 	pos_acc: 0
[listener-2] 	status: 0
[listener-2] imu:
[listener-2] 	acc_x: 0.000881995656527579
[listener-2] 	acc_y: 0.010707189328968525
[listener-2] 	acc_z: -0.998849987983703613
[listener-2] 	gyro_x: -0.000280264881439507
[listener-2] 	gyro_y: 0.034663043916225433
[listener-2] 	gyro_z: -0.007341515272855759
[listener-2] 	temp: 36.0000
[listener-2] ---

```




