# ROS  can例程

本文档介绍如何在ROS下IMU的数据，并提供了c++语言例程代码，通过执行ROS命令，运行相应的节点，就可以看到打印到终端上的信息。

* 测试环境：Ubuntu20.04

* ROS版本：ROS_noetic


* 测试设备：AHRS200



## 安装socketcan_bridge和ros_canopen
为了在ROS中使用SocketCAN，你需要`socketcan_bridge`包和`ros_canopen`包，这些包提供了ROS节点以接收和发送CAN帧的功能。

注意不同的ros版本安装安装socketcan_bridge和ros_canopen命令不一样
1. noetic版本ros（适用于Ubuntu 20.04）
```bash
sudo apt-get update
sudo apt-get install ros-noetic-socketcan-bridge
sudo apt-get install ros-noetic-ros-canopen
```

2. Kinetic版本ros（适用于Ubuntu 16.04）

```bash
sudo apt-get update
sudo apt-get install ros-kinetic-socketcan-bridge
sudo apt-get install ros-kinetic-ros-canopen
```
3. Melodic版本ros（适用于Ubuntu 18.04）

```bash
sudo apt-get update
sudo apt-get install ros-melodic-socketcan-bridge
sudo apt-get install ros-melodic-ros-canopen
```
   

## 配置CAN接口

需要设置并启动CAN接口。打开终端，使用以下命令配置并启动你的can0接口（确保使用正确的设备名，如果是其他接口，如can1，则需相应调整）
```bash
sudo ip link set can0 down
sudo ip link set can0 up type can bitrate 500000
```
这里设置的比特率是500kbps，根据你的CAN网络实际情况进行调整。



## 修改CAN接口名字
当前驱动代码默认CAN接口名字can0。
如需要修改，请在can_imu.cpp文件下修改宏定义：

```shell
#define CAN_INTERFACE_NAME "can0"
```


## 编译代码

cd can_imu_ros1
catkin_make
source devel/setup.bash



###  查看 IMU 数据

1. 打开一个终端，执行：
roscore
回到can_imu_ros1⽂件夹下 执⾏
```shell
source devel/setup.bash
```
执⾏启动 rosrun
```shell
rosrun can_imu can_imu
```
2. 打开新的终端:
```shell
source devel/setup.bash
```
输⼊命令查看 IMU 数据
```shell
rostopic echo IMU_data
```
```shell
header: 
  seq: 2102
  stamp: 
    secs: 1712122898
    nsecs: 699805721
  frame_id: "imu"
orientation: 
  x: -0.0010773062987748604
  y: 0.016450232956337168
  z: -0.10690676696280528
  w: 0.9941323716811465
orientation_covariance: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
angular_velocity: 
  x: 0.00010886170027006417
  y: 0.0020122985332117972
  z: -0.000609063357935883
angular_velocity_covariance: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
linear_acceleration: 
  x: -0.31544684584792704
  y: -0.04979020018458832
  z: 9.545999998950958
linear_acceleration_covariance: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
```
默认输出的线加速度(linear acceleration)单位为m/s^2，
输出角加速度(angular velocity)单位为rad/s，
输出方向(orientation)为 四元数(Quaternion) 


3. 使⽤rviz 查看 IMU 数据
⾸先安装 imu_tool
sudo apt-get install ros-noetic-imu-tools
打开 rviz，选择 Fixed_Frame 为 imu，Add 添加 imu，且 Topic 为/IMU_data，则可以显
示三轴姿态。

###  如何修改坐标系

默认坐标系为前右下，需要修改坐标系，需要在serial_imu.cpp文件下代码初始化地方添加如下代码：
```shell
Send_CMD_LONG(14,101,0,4,0,0,0); //设置IMU坐标系，默认为101，前右下
for (int i = 0; i < 10; i++)
can_send_data((uint8_t *)&data_cmd_long,sizeof(data_cmd_long));
```
其中Send_CMD_LONG函数里第二个参数101为默认朝向，第一个参数14与第三个参数4为固定值。
如需要修改朝向更改第二个参数即可，参数值参考下表：

```shell
101 +Ux +Uy +Uz 默认朝向
102 -Ux -Uy +Uz
103 -Uy +Ux +Uz
104 +Uy -Ux +Uz
105 -Ux +Uy -Uz
106 +Ux -Uy -Uz
107 +Uy +Ux -Uz
108 -Uy -Ux -Uz
109 -Uz +Uy +Ux
110 +Uz -Uy +Ux
111 +Uy +Uz +Ux
112 -Uy -Uz +Ux
113 +Uz +Uy -Ux
114 -Uz -Uy -Ux
115 -Uy +Uz -Ux
116 +Uy -Uz -Ux
117 -Ux +Uz +Uy
118 +Ux -Uz +Uy
119 +Uz +Ux +Uy
120 -Uz -Ux +Uy
121 +Ux +Uz -Uy
122 -Ux -Uz -Uy
123 -Uz +Ux -Uy
124 +Uz -Ux -Uy
```

###  如何查看发布topic的频率

输入以下命令可以查看topic频率
```shell
rostopic hz /IMU_data
```

输入命令后可以看到数据频率在100hz左右
```shell
average rate: 100.510
	min: 0.0098s max: 0.0102s std dev: 0.02638s window: 1400
average rate: 100.469
	min: 0.0098s max: 0.0102s std dev: 0.02639s window: 1504
```

###  录制rosbag并回放

1.录制rosbag
打开串口后，在新的终端中输入以下命令
```shell
rosbag record /IMU_data
```
当完成所需数据的录制后，按Ctrl+C终止rosbag record命令，rosbag文件将自动保存。

2.回放rosbag

 在新的终端窗口中，使用rosbag play命令回放之前录制的rosbag文件。例如，如果您要回放名为my_bag.bag的文件，执行：
```shell
rosbag play my_bag.bag
```

在另一个终端中，可以使用rostopic echo或rqt等工具监控回放的话题数据，确保数据正确回放,输入以下命令:
```shell
rostopic echo /IMU_data
```
在回放期间，相应的ROS节点应能接收到回放的数据，如同实时数据一样。观察您的应用程序或可视化工具是否正确响应这些数据。
按Ctrl+C终止rosbag play命令，停止回放



























 

