#include <iostream>
#include <sensor_msgs/msg/imu.hpp>
#include "rclcpp/rclcpp.hpp"
#include "nav619_ros2/msg/nav.hpp"
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include "serial_parse.h"
#include <linux/serial.h> // 新增头文件
#include <sys/ioctl.h>	  // 新增头文件
#ifdef __cplusplus
extern "C"
{
#endif
#include <poll.h>
#define GRA_ACC (9.8)
#define DEG_TO_RAD (0.01745329)
#define BUF_SIZE (1024)
#ifdef __cplusplus
}
#endif

using namespace std::chrono_literals;
using namespace std;
extern struct MULTI_LONG_CMD_STRUCT data_cmd_long;

std::string serial_port;
int baud_rate;
std::string frame_id;
std::string imu_topic;
nav619_ros2::msg::Nav nav_msg;
rclcpp::Publisher<nav619_ros2::msg::Nav>::SharedPtr nav_pub;

class IMUPublisher : public rclcpp::Node
{
public:
	int fd = 0;
	uint8_t buf[BUF_SIZE] = {0};
	IMUPublisher() : Node("IMU_publisher")
	{
		this->declare_parameter<std::string>("serial_port", "/dev/ttyUSB0");
		this->declare_parameter<int>("baud_rate", 460800);
		this->declare_parameter<std::string>("frame_id", "base_link");
		this->declare_parameter<std::string>("imu_topic", "/nav619_data");

		this->get_parameter("serial_port", serial_port);
		this->get_parameter("baud_rate", baud_rate);
		this->get_parameter("frame_id", frame_id);
		this->get_parameter("imu_topic", imu_topic);

		RCLCPP_INFO(this->get_logger(), "serial_port: %s\r\n", serial_port.c_str());
		RCLCPP_INFO(this->get_logger(), "baud_rate: %d\r\n", baud_rate);
		RCLCPP_INFO(this->get_logger(), "frame_id: %s\r\n", frame_id.c_str());
		RCLCPP_INFO(this->get_logger(), "imu_topic: %s\r\n", imu_topic.c_str());

		nav_pub = this->create_publisher<nav619_ros2::msg::Nav>(imu_topic, 10);
		// 配置低延迟模式
		if (configure_low_latency(serial_port))
		{
			RCLCPP_ERROR(this->get_logger(), "Failed to configure low latency mode");
			rclcpp::shutdown();
		}
		fd = open_serial(serial_port, baud_rate);
		Send_CMD_LONG(3, 8, 0, 0, 0, 0, 0);
		for (int i = 0; i < 10; i++)
			write(fd, (uint8_t *)&data_cmd_long, sizeof(data_cmd_long));

		Send_CMD_LONG(5, 0, 0, 0, 0, 0, 0);
		for (int i = 0; i < 3; i++)
			write(fd, (uint8_t *)&data_cmd_long, sizeof(data_cmd_long));

		Send_CMD_LONG(3, 8, 0, 0, 0, 0, 0);
		for (int i = 0; i < 10; i++)
			write(fd, (uint8_t *)&data_cmd_long, sizeof(data_cmd_long));

		while (1)
			imu_read();
	}

private:
	void imu_read(void)
	{
		struct pollfd p;
		p.fd = fd;
		p.events = POLLIN;

		int rpoll = poll(&p, 1, 5);

		if (rpoll == 0)
			return;

		int n = read(fd, buf, sizeof(buf));
		for (int i = 0; i < n; i++)
		{
			imu_rx(buf[i]);
		}

		memset(buf, 0, sizeof(buf));
	}

private:
	// 新增低延迟配置函数
	int configure_low_latency(const std::string &port)
	{
		int tmp_fd = open(port.c_str(), O_RDWR | O_NOCTTY);
		if (tmp_fd < 0)
		{
			RCLCPP_ERROR(this->get_logger(), "Error opening serial port: %s", strerror(errno));
			return -1;
		}

		struct serial_struct ser_info;
		if (ioctl(tmp_fd, TIOCGSERIAL, &ser_info) < 0)
		{
			RCLCPP_ERROR(this->get_logger(), "Error getting serial info: %s", strerror(errno));
			close(tmp_fd);
			return -1;
		}

		ser_info.flags |= ASYNC_LOW_LATENCY;

		if (ioctl(tmp_fd, TIOCSSERIAL, &ser_info) < 0)
		{
			RCLCPP_ERROR(this->get_logger(), "Error setting low latency mode: %s", strerror(errno));
			close(tmp_fd);
			return -1;
		}

		close(tmp_fd);
		RCLCPP_INFO(this->get_logger(), "Low latency mode enabled for port: %s", port.c_str());
		return 0;
	}

	int open_serial(std::string port, int baud)
	{
		const char *port_device = port.c_str();
		// 修改打开标志：移除O_NONBLOCK，增加O_SYNC
		int fd = open(port_device, O_RDWR | O_NOCTTY | O_SYNC);

		if (fd == -1)
		{
			RCLCPP_ERROR(this->get_logger(), "Unable to open serial port: %s", strerror(errno));
			exit(0);
		}

		if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
			RCLCPP_ERROR(this->get_logger(), "fcntl failed: %s", strerror(errno));
		else
			fcntl(fd, F_SETFL, O_NONBLOCK);

		struct termios options;
		memset(&options, 0, sizeof(options));
		tcgetattr(fd, &options);

		switch (baud)
		{
		case 115200:
			cfsetispeed(&options, B115200);
			cfsetospeed(&options, B115200);
			break;
		case 460800:
			cfsetispeed(&options, B460800);
			cfsetospeed(&options, B460800);
			break;
		case 921600:
			cfsetispeed(&options, B921600);
			cfsetospeed(&options, B921600);
			break;
		}

		options.c_cflag &= ~PARENB;
		options.c_cflag &= ~CSTOPB;
		options.c_cflag &= ~CSIZE;
		options.c_cflag |= HUPCL;
		options.c_cflag |= CS8;
		options.c_cflag &= ~CRTSCTS;
		options.c_cflag |= CREAD | CLOCAL;

		options.c_iflag &= ~(IXON | IXOFF | IXANY);
		options.c_iflag &= ~(INLCR | ICRNL);

		options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

		options.c_oflag &= ~OPOST;
		options.c_oflag &= ~(ONLCR | OCRNL);

		options.c_cc[VMIN] = 0;
		options.c_cc[VTIME] = 0;
		tcsetattr(fd, TCSANOW, &options);

		return fd;
	}

	// void imu_write(const uint8_t *data, size_t size)
	// {
	// 	if (data == NULL || size == 0) {
	// 	// 没有数据需要写入，或者提供的数据大小为0
	// 	return;
	// 	}

	// 	ssize_t n_written = 0;
	// 	ssize_t total_written = 0;

	// 	while (total_written < size) {
	// 	n_written = write(fd, data + total_written, size - total_written);
	// 	if (n_written < 0) {
	// 	break;
	// 	}
	// 	total_written += n_written;
	// 	}

	// }
};

int main(int argc, const char *argv[])
{
	rclcpp::init(argc, argv);
	rclcpp::spin(std::make_shared<IMUPublisher>());
	rclcpp::shutdown();

	return 0;
}
