# NAV680d_ROS串口例程

本文档介绍如何在ROS下读取nav680d的数据，并提供了c++语言例程代码，通过执行ROS命令，运行相应的节点，就可以看到打印到终端上的信息。

* 测试环境：Ubuntu20.04

* ROS版本：ROS_noetic


* 测试设备：nav680d

## 安装 ROS serial 软件包，

安装 ROS serial 软件包，本例程依赖 ROS 提供的 serial 包实现串口通信。
首先执行如下命令，下载安装 serial 软件包：
sudo apt install ros-noetic-serial


如果出现报错，如pkg was interrupted, you must manually run 'sudo dpkg --configure -a' to correct the problem，则进行以下命令解决：
sudo dpkg --configure -a
sudo apt update

注意不同的ros版本安装ROS serial 的命令不一样

1. Kinetic版本ros（适用于Ubuntu 16.04）

   ```bash
   sudo apt-get update
   sudo apt-get install ros-kinetic-rosserial
   ```

2. Melodic版本ros（适用于Ubuntu 18.04）

   ```bash
   sudo apt-get update
   sudo apt-get install ros-melodic-rosserial
   ```
## 安装USB-UART驱动

Ubuntu 默认不需要安装串口驱动。将USB线连接到电脑上时，会自动识别设备。识别成功后，会在dev目录下出现一个对应的设备:ttyUSBx

检查USB-UART设备是否被Ubantu识别：

1. 打开终端，输入`ls /dev`,先查看已经存在的串口设备。
2. 查看是否已经存在  ttyUSBx 这个设备文件，便于确认对应的端口号。
4. 接下来插入USB线，连接调试板，然后再次执行`ls /dev`。 dev目录下多了一个设备`ttyUSB0`：

```shell
linux@ubuntu:~$ ls /dev
.....
hpet             net           tty11     tty4   ttyS0      ttyUSB0    vhost-vsock
hugepages        null          tty12     tty40  ttyS1      udmabuf  vmci
......
```

4.打开USB设备的可执行权限：

```shell
   $ sudo chmod 777 /dev/ttyUSB0
```

## 设置串口低延迟
5.为了保证数据传输的实时性，需要设置串口为低延迟
```shell
  安装: sudo apt install setserial
 插入usb线后执行:  setserial /dev/ttyUSB0 low_latency
   ttyUSB0换成实际使用的串口
```


## 修改串口号与波特率
当前驱动代码默认波特率为460800，默认串口号为ttyUSB0。
如需要修改波特率与串口号，请在forsense_ins.cpp文件下修改宏定义：

```shell
#define IMU_SERIAL   "/dev/ttyUSB0"
#define BAUD         (115200)
#define BUF_SIZE     1024
```
注意:ROS驱动代码里的波特率需要与我们产品设备波特率保持一致，


## 编译代码
```shell
//进入工作文件夹
cd nav680d_ros1
//编译
catkin_make

//执⾏载入工作空间的环境设置
source devel/setup.bash
```


###  查看 IMU 数据

1. 打开一个终端，执行：
```shell
roscore
```
2.回到nav680d_ros1⽂件夹下,在新窗口下输入以下命令:
```shell
 //执⾏载入工作空间的环境设置
source devel/setup.bash

执⾏启动 rosrun,打开串口
rosrun forsense_ins forsense_ins
```

3. 打开新的终端,输入以下命令可获取数据:
```shell
//再次执行执⾏载入工作空间的环境设置:
source devel/setup.bash

//列出当前运行系统中所有活跃的话题
rostopic list

输⼊命令查看组合导航数据
rostopic echo /nav680Data
```


###  如何查看发布topic的频率

输入以下命令可以查看topic频率
```shell
rostopic hz /nav680Data
```

输入命令后可以看到数据频率在100hz左右
```shell
average rate: 100.510
	min: 0.0098s max: 0.0102s std dev: 0.02638s window: 1400
average rate: 100.469
	min: 0.0098s max: 0.0102s std dev: 0.02639s window: 1504
```

###  录制rosbag并回放

1.录制rosbag
打开串口后，在新的终端中输入以下命令
```shell
rosbag record /nav680Data
```
当完成所需数据的录制后，按Ctrl+C终止rosbag record命令，rosbag文件将自动保存。

2.回放rosbag

 在新的终端窗口中，使用rosbag play命令回放之前录制的rosbag文件。例如，如果您要回放名为my_bag.bag的文件，执行：
```shell
rosbag play my_bag.bag
```

在另一个终端中，可以使用rostopic echo或rqt等工具监控回放的话题数据，确保数据正确回放,输入以下命令:
```shell
rostopic echo /IMU_data
```
在回放期间，相应的ROS节点应能接收到回放的数据，如同实时数据一样。观察您的应用程序或可视化工具是否正确响应这些数据。
按Ctrl+C终止rosbag play命令，停止回放




