#include "main.h"
#include "gpio.h"
#include "spi.h"
#include "imu_drv.h"
#include "math.h"


#define G_Data 0.00980665
#define PI 3.141592653589793
#define DUMMY 0x00FF
uint32_t spi_error=0;
uint16_t filter_ctrl;
uint16_t imu_id[3];
uint16_t checksum_spi_calc;
uint32_t all_count_SPI;




typedef struct {
    float  gyro[3];
    float  accel[3];
    uint16_t checksum;
}spi_burst_data_t __attribute__((packed));
spi_burst_data_t  spi_burst_data;


typedef struct {
    float  gyro[3];
    float  accel[3];
    float temp;
}spi_regs_data_t __attribute__((packed));
spi_regs_data_t  spi_regs_data;



typedef struct{
	float gyro[3];
	float accel[3];
	float temperature;
	float attitude[3];
	uint32_t status;
	uint32_t check_crc;
}spi_burst_ext_data_t_mode2;
spi_burst_ext_data_t_mode2 spi_burst_data_mode2;


typedef struct{
	uint16_t time;
	uint16_t status;
	float gyro[3];
  float accel[3];
	float attitude[3];
	float temperature;
	float qnq[4];
	uint32_t check_crc;
}spi_burst_ext_data_t_mode3 __attribute__((packed));
spi_burst_ext_data_t_mode3 spi_burst_data_mode3;




/**
 * IMU的SPI主机接口初始化
 */
void IMU_Reboot(void)
{
    HAL_GPIO_WritePin(RST_IMU_GPIO_Port, RST_IMU_Pin, GPIO_PIN_SET);
    HAL_Delay(100);
    HAL_GPIO_WritePin(RST_IMU_GPIO_Port, RST_IMU_Pin, GPIO_PIN_RESET);
    HAL_Delay(100);
    HAL_GPIO_WritePin(RST_IMU_GPIO_Port, RST_IMU_Pin, GPIO_PIN_SET); 
    HAL_Delay(3000);
}


/**
 * SPI双向传输数据
 */
uint16_t SPI1_ReadWriteByte16(uint16_t cmd)
{
    uint16_t rev;
    HAL_SPI_TransmitReceive(&hspi1,(uint8_t* )&cmd,(uint8_t* )&rev,1,0xFFFFFF); 

    return rev;
}





/**
 * SPI读取单个寄存器
 */
uint16_t SPI1_ReadReg(uint16_t addr)
{
    SPI1_ReadWriteByte16(addr);
    SPI1_ReadWriteByte16(DUMMY);
    return SPI1_ReadWriteByte16(DUMMY);
}


/**
 * 初始化SPI寄存器，标准帧
 */
void SPI_Master_Init()
{
    IMU_Reboot();
    SPI1_ReadWriteByte16(0xFE01);//选择配置窗口
    SPI1_ReadWriteByte16(0x8655);//滤波器设置,15hz,数值参考手册
    filter_ctrl = SPI1_ReadReg(0x0600);

    imu_id[0] = SPI1_ReadReg(0x6C00);
    imu_id[1] = SPI1_ReadReg(0x6E00);
    imu_id[2] = SPI1_ReadReg(0x7000);

    SPI1_ReadWriteByte16(0xFE00);//切换为数据窗口
//	SPI1_ReadWriteByte16(0x8D00);//读取Burst模式（默认）
//	SPI1_ReadWriteByte16(0x8D02);//读取Burst模式2
//	SPI1_ReadWriteByte16(0x8D03);//读取Burst模式3

}

/**
 * SPI读取-burst方式
 */
void SPI_Master_Read_Burst()
{
	uint16_t imu_data[13]={0};
	
	uint16_t cmd[2] = {0x8000, 0x0000};
    HAL_SPI_Transmit(&hspi1, (uint8_t *)cmd, 2, 2);

	/**
     * 读取整帧共30个uint16
     * 使用一次性读取所有数据
     */
		uint16_t dummy_data[13] = {0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x0000,0x0000,0x0000};  // 占位符
		HAL_SPI_TransmitReceive(&hspi1, (uint8_t *)dummy_data, (uint8_t *)imu_data, sizeof(imu_data)/2,2);  // 超时时间为2ms
		all_count_SPI++;

    /**
     * 和校验
     */
    checksum_spi_calc = 0;
    for(int i=0;i<sizeof(spi_burst_data_t)/2-2;i++)
		{
    checksum_spi_calc+=imu_data[i];
		spi_burst_data.checksum=checksum_spi_calc;
		}
    
    /**
     * 小端模式
     */
    if(imu_data[12]==checksum_spi_calc)
    {
        spi_burst_data.gyro[0] = (float)((int32_t)(imu_data[1]<<16) | imu_data[0])/4096000.0f;
        spi_burst_data.gyro[1] = (float)((int32_t)(imu_data[3]<<16) | imu_data[2])/4096000.0f;
        spi_burst_data.gyro[2] = (float)((int32_t)(imu_data[5]<<16) | imu_data[4])/4096000.0f;

        spi_burst_data.accel[0] = (float)((int32_t)(imu_data[7]<<16) | imu_data[6])/327680.0f;
        spi_burst_data.accel[1] = (float)((int32_t)(imu_data[9]<<16) | imu_data[8])/327680.0f;
        spi_burst_data.accel[2] = (float)((int32_t)(imu_data[11]<<16) | imu_data[10])/327680.0f;

				

    }
    else
    {
        spi_error++;
    }
}

/**
*SPI读取burst2方式
*/
uint32_t data_crc=0;
uint32_t data_u32=0;
float data=0;
void SPI_Master_Read_Burst_ext()
{
	uint16_t imu_data[24]={0};
	
	uint16_t cmd[2] = {0x8000, 0x0000};
    HAL_SPI_Transmit(&hspi1, (uint8_t *)cmd, 2, 2);

	/**
     * 读取整帧共30个uint16
     * 使用一次性读取所有数据
     */
		uint16_t dummy_data[24] = {0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000};  // 占位符
		HAL_SPI_TransmitReceive(&hspi1, (uint8_t *)dummy_data, (uint8_t *)imu_data, sizeof(imu_data)/2,2);  // 超时时间为2ms
		all_count_SPI++;
	
	/**
	*CRC校验
	*/
	uint32_t check_crc=1;
	data_crc=crc_crc32(check_crc,(uint8_t *)(imu_data),sizeof(spi_burst_ext_data_t_mode2)-4);

	/**
	*小端模式
	*/
	if((uint32_t)(imu_data[23]<<16|imu_data[22])==data_crc)
		{
			data_u32 = (imu_data[1]<<16|imu_data[0]);
			spi_burst_data_mode2.gyro[0] = *(float*)&data_u32;
			
			data_u32 = (imu_data[3]<<16|imu_data[2]);
			spi_burst_data_mode2.gyro[1] = *(float*)&data_u32;
			
			data_u32 = (imu_data[5]<<16|imu_data[4]);
			spi_burst_data_mode2.gyro[2] = *(float*)&data_u32;
			
			data_u32 = (imu_data[7]<<16|imu_data[6]);
			spi_burst_data_mode2.accel[0] = *(float*)&data_u32;
			
			data_u32 = (imu_data[9]<<16|imu_data[8]);
			spi_burst_data_mode2.accel[1] = *(float*)&data_u32;
			
			data_u32 = (imu_data[11]<<16|imu_data[10]);
			spi_burst_data_mode2.accel[2] = *(float*)&data_u32;
			
			data_u32 = (imu_data[13]<<16|imu_data[12]);
			spi_burst_data_mode2.temperature = *(float*)&data_u32;
			
			data_u32 = (imu_data[15]<<16|imu_data[14]);
			spi_burst_data_mode2.attitude[0] = *(float*)&data_u32;
			
			data_u32 = (imu_data[17]<<16|imu_data[16]);
			spi_burst_data_mode2.attitude[1] = *(float*)&data_u32;
			
			data_u32 = (imu_data[19]<<16|imu_data[18]);
			spi_burst_data_mode2.attitude[2] = *(float*)&data_u32;
			
			spi_burst_data_mode2.status=((int32_t)(imu_data[21]<<16)|imu_data[20]);
			
			
			spi_burst_data_mode2.check_crc =((int32_t)(imu_data[23]<<16)|imu_data[22]);
		}
		else
		{
			spi_error++;
		}
}


/**
*SPI读取burst3方式
*/

void SPI_Master_Read_Burst_ext3()
{
	uint16_t imu_data[32]={0};
	
	
		    /**
     * 偏移2位读取数据
     */
    uint16_t cmd[2] = {0x8000, 0x0000};
    HAL_SPI_Transmit(&hspi1, (uint8_t *)cmd, 2, 2);

	/**
     * 读取整帧共30个uint16
     * 使用一次性读取所有数据
     */
		uint16_t dummy_data[32] = {0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
		,0x0000,0x0000};  // 占位符
		HAL_SPI_TransmitReceive(&hspi1, (uint8_t *)dummy_data, (uint8_t *)imu_data, sizeof(imu_data)/2,2);  // 超时时间为2ms
		all_count_SPI++;
	
	/**
	*CRC校验
	*/
	uint32_t check_crc=1;
	data_crc=crc_crc32(check_crc,(uint8_t *)(imu_data),sizeof(spi_burst_ext_data_t_mode3)-4);
	
 
 
	
/**
	*小端模式
	*/
	if((uint32_t)(imu_data[31]<<16|imu_data[30])==data_crc)
		{
			data_u32 = (imu_data[0]);
			spi_burst_data_mode3.time = *(uint16_t*)&data_u32;
			
			data_u32 = (imu_data[1]);
			spi_burst_data_mode3.status = *(uint16_t*)&data_u32;

			data_u32 = (imu_data[3]<<16|imu_data[2]);
			spi_burst_data_mode3.gyro[0] = *(float*)&data_u32;

			data_u32 = (imu_data[5]<<16|imu_data[4]);
			spi_burst_data_mode3.gyro[1] = *(float*)&data_u32;

			data_u32 = (imu_data[7]<<16|imu_data[6]);
			spi_burst_data_mode3.gyro[2] = *(float*)&data_u32;

			data_u32 = (imu_data[9]<<16|imu_data[8]);
			spi_burst_data_mode3.accel[0] = *(float*)&data_u32;

			data_u32 = (imu_data[11]<<16|imu_data[10]);
			spi_burst_data_mode3.accel[1] = *(float*)&data_u32;

			data_u32 = (imu_data[13]<<16|imu_data[12]);
			spi_burst_data_mode3.accel[2] = *(float*)&data_u32;

			data_u32 = (imu_data[15]<<16|imu_data[14]);
			spi_burst_data_mode3.attitude[0] = *(float*)&data_u32;

			data_u32 = (imu_data[17]<<16|imu_data[16]);
			spi_burst_data_mode3.attitude[1] = *(float*)&data_u32;

			data_u32 = (imu_data[19]<<16|imu_data[18]);
			spi_burst_data_mode3.attitude[2] = *(float*)&data_u32;

			data_u32 = (imu_data[21]<<16|imu_data[20]);
			spi_burst_data_mode3.temperature = *(float*)&data_u32;



		  data_u32 = (imu_data[23]<<16|imu_data[22]);
			spi_burst_data_mode3.qnq[0] = *(float*)&data_u32;

			data_u32 = (imu_data[27]<<16|imu_data[26]);
			spi_burst_data_mode3.qnq[1] = *(float*)&data_u32;

			data_u32 = (imu_data[25]<<16|imu_data[24]);
			spi_burst_data_mode3.qnq[2] = *(float*)&data_u32;

			data_u32 = (imu_data[29]<<16|imu_data[28]);
			spi_burst_data_mode3.qnq[3] = *(float*)&data_u32;


			spi_burst_data_mode3.check_crc =((int32_t)(imu_data[31]<<16)|imu_data[30]);

		}
		else
		{
			spi_error++;
		}
}


/**
 * SPI读取-寄存器方式
 */
void SPI_Master_Read_Regs()
{
    uint16_t imu_data[30]={0}; 
    /**
     * 偏移2位读取数据
     */
    imu_data[0]  = SPI1_ReadWriteByte16(0x1200);
    imu_data[1]  = SPI1_ReadWriteByte16(0x1400);
    imu_data[2]  = SPI1_ReadWriteByte16(0x1600);
    imu_data[3]  = SPI1_ReadWriteByte16(0x1800);
    imu_data[4]  = SPI1_ReadWriteByte16(0x1A00);
    imu_data[5]  = SPI1_ReadWriteByte16(0x1C00);
    imu_data[6]  = SPI1_ReadWriteByte16(0x1E00);
    imu_data[7]  = SPI1_ReadWriteByte16(0x2000);
    imu_data[8]  = SPI1_ReadWriteByte16(0x2200);
    imu_data[9]  = SPI1_ReadWriteByte16(0x2400);
    imu_data[10]  = SPI1_ReadWriteByte16(0x2600);
    imu_data[11]  = SPI1_ReadWriteByte16(0x2800);
    imu_data[12]  = SPI1_ReadWriteByte16(0x0e00);
    imu_data[13]  = SPI1_ReadWriteByte16(0x1000);
    imu_data[14]  = SPI1_ReadWriteByte16(DUMMY);
    imu_data[15]  = SPI1_ReadWriteByte16(DUMMY);


    spi_regs_data.gyro[0] = (float)((int32_t)(imu_data[2]<<16) | imu_data[3])/4096000.0f;
    spi_regs_data.gyro[1] = (float)((int32_t)(imu_data[4]<<16) | imu_data[5])/4096000.0f;
    spi_regs_data.gyro[2] = (float)((int32_t)(imu_data[6]<<16) | imu_data[7])/4096000.0f;

    spi_regs_data.accel[0] = (float)((int32_t)(imu_data[8]<<16) | imu_data[9])/327680000.0f;
    spi_regs_data.accel[1] = (float)((int32_t)(imu_data[10]<<16) | imu_data[11])/327680000.0f;
    spi_regs_data.accel[2] = (float)((int32_t)(imu_data[12]<<16) | imu_data[13])/327680000.0f;


    spi_regs_data.temp = -((float)(((imu_data[14]<<16) | imu_data[15]) - 172621824))/263.4/65536+25;
}
